//*********************************************************
// Ez a unit kapja az enginetol az eventeket.
// az esemny bekvetkezse utn postmessage-gel kell
// tovbbadni a vezrlst az implementcinak, hogy
// nehogy deadlock-ba vigyk az engine-t.
//*********************************************************
unit unMain;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs,unImpl,unXMLSettingsForm,unenvelope;

const
  LOG_LIFE = 1;
  CONNECTED = 1;
  DISCONNECTED =0;
  SOFF = 0;
  SON = 1;
  SNOW = 2;

type

  TEnvelopeEvent = procedure (envelopeXML : string) of object;

  TfrmMain = class(TObject)
  private
    { Private declarations }
  public
    { Public declarations }
    pluginName    : string;
    instanceName  : string;
    messageEngine : pointer;
    exedir        : string;
    engineName    : string;
    regkey        : string;
    desiredState  : integer;
    connState     : integer;
    reconnState   : integer;
    impl          : TPluginImpl;
    Pref          : TXmlSettingsForm;

    implEnvelopeReceived  : TEnvelopeEvent;
    implEnvelopeSent      : TEnvelopeEvent;
    implEnvelopeDelivered : TEnvelopeEvent;
    implEnvelopeDeleted   : TEnvelopeEvent;

    //LifeCycle Management
    constructor create;
    procedure   onInit;
    procedure   onUninstall;
    destructor  destroy; override;

    procedure   onAutoConnect;
    procedure   onConnect;
    procedure   onDisconnect;

    //Methods
    procedure lock;
    procedure unlock;
    procedure registerMenuItem(itemName : string);
    procedure unRegisterMenuItem(itemName : string);
    procedure sendEnvelope(envelopeXML : string);
    function  ask(target,msg:string) : string;
    procedure work;

    //Events
    procedure onEnvelopeReceived(envelopeXML : string);
    procedure onEnvelopeSent(envid:string);
    procedure onEnvelopeDelivered(envid:string);
    procedure onEnvelopeDeleted(envid:string);
    function  onGetForm(formURI:string) : string;
    function  onPostFormData(formURI,postdata:string) : string;
    procedure signalConnected;
    procedure signalNotConnected;
    function sendInfo(action,param:string):string;
    function getMessageById(id:string):TEnvelope;

    //Processing functions
    procedure addtolog(event : string);
    procedure addtodirectlog(msg : widestring);

  end;

implementation
uses unPlugin,unStrings,unDate;

constructor TfrmMain.create;
Begin
   desiredState := DISCONNECTED;
   connState := DISCONNECTED;
   reconnState := SOFF;
   pref := nil;
End;

destructor TfrmMain.destroy;
Begin
   //
End;

procedure TfrmMain.oninit;
var filename : string;
Begin
    exedir := ask('getSystemInfo','exedir');
    engineName := ask('getSystemInfo','engineName');

    regkey := 'SOFTWARE\Ozeki\'+engineName+'\Plugins\'+instanceName;
    pluginName := instanceName;
    while isNumber(pluginName[length(pluginName)]) do pluginName := chopEnd(pluginName,1);

    pluginName := strreplace(' ','',trim(pluginName));


    //Load the preferences;
    filename := exedir+'Forms\'+PluginName+'_frmPreferences.xml';
    if not fileexists(filename) then addtolog('Cannot find preferences form: '+filename);
    Pref     := TXmlSettingsForm.Create(regkey,filename);
    Pref.settings.setVal('ePluginName',pluginName);
    pref.caption := instanceName+' configuration';
    registerMenuItem(instanceName+' configuration');

    //Initialize the implemenation;
    impl       := nil;
    impl       := TPluginImpl.Create;
    impl.owner := self;
    impl.init;

    if assigned(impl) then TPlugin(messageEngine).setInstanceDescr(impl.getNiceName);
End;

procedure TfrmMain.onUninstall;
Begin
    unRegisterMenuItem(instanceName+' configuration');
    if assigned(impl) then impl.Free;
    pref.free;
End;


//********************************************************************
// procedures you can use
//********************************************************************

procedure TfrmMain.work;
Begin
 if assigned(messageEngine) then TPlugin(messageEngine).work;
End;

procedure TfrmMain.lock;
Begin
  if assigned(messageEngine) then TPlugin(messageEngine).lock;
End;

procedure TfrmMain.unlock;
Begin
  if assigned(messageEngine) then TPlugin(messageEngine).unlock;
End;

procedure TfrmMain.registerMenuItem(itemName : string);
Begin
  if assigned(messageEngine) then TPlugin(messageEngine).registerMenuItem(itemName);
End;

procedure TfrmMain.unRegisterMenuItem(itemName : string);
Begin
  if assigned(messageEngine) then TPlugin(messageEngine).unRegisterMenuItem(itemName);
End;

procedure TfrmMain.sendEnvelope(envelopeXML : string);
Begin
  if assigned(messageEngine) then TPlugin(messageEngine).sendEnvelope(envelopeXML);
End;

function  TfrmMain.ask(target,msg:string) : string;
Begin
  if assigned(messageEngine) then Result := TPlugin(messageEngine).pluginInfo(target,msg);
End;

function TfrmMain.getMessageById(id:string):TEnvelope;
var ret : TEnvelope;
    envxml : string;
Begin
  if assigned(messageEngine) then envxml := TPlugin(messageEngine).pluginInfo('getMessageById',id);
  ret := nil;
  if (envxml<>'unknown') and (envxml<>'notHandled') then begin
     ret := TEnvelope.Create;
     ret.fromXml(envxml);
  end;
  result := ret;
End;


//********************************************************************
// Events received from the SMS engine
//********************************************************************

procedure TfrmMain.onEnvelopeReceived(envelopeXML : string);
var operator : string;
    env : TEnvelope;
    goon : boolean;
Begin
   if (desiredState = CONNECTED) and assigned(impl) and assigned(implEnvelopeReceived) then begin
        operator := pref.settings.getVal('eOperatorName');
        goon := true;
        if operator<>'' then begin
           env := TEnvelope.Create;
           env.fromXml(envelopeXml);
           if env.operator<>operator then begin
              goon := false;
              addtolog('Envelope filtered. Operator names do not match: ['+env.operator+']['+operator+']');
           end;
           env.free;
        end;
        if goon then implEnvelopeReceived(envelopeXML);
   end;
End;

procedure TfrmMain.onEnvelopeSent(envid:string);
Begin
   if (desiredState = CONNECTED) and assigned(implEnvelopeSent) and assigned(impl) then begin
         implEnvelopeSent(envid);
   end;
End;                                                  

procedure TfrmMain.onEnvelopeDelivered(envid:string);
Begin
   if (desiredState = CONNECTED) and assigned(implEnvelopeDelivered) and assigned(impl) then begin
      implEnvelopeDelivered(envid);
   end;
End;                            

procedure TfrmMain.onEnvelopeDeleted(envid:string);
Begin
   if (desiredState = CONNECTED) and assigned(implEnvelopeDeleted) and assigned(impl) then begin
      implEnvelopeDeleted(envid);
   end;
End;

function  TfrmMain.onGetForm(formURI:string) : string;
Begin
  Result := pref.getForm;
End;

function  TfrmMain.onPostFormData(formURI,postdata:string) : string;
Begin
  Result := pref.postFormData(postdata);
End;

procedure TfrmMain.onAutoConnect;
Begin
  if assigned(pref) and
     (pref.settings.checked('cbAutoconnect') or
     pref.settings.checked('cbAutostart'))
      then begin
     onConnect;
  end;
End;

procedure TfrmMain.onConnect;
Begin
  if connState<>CONNECTED then begin
     desiredState := CONNECTED;
     reconnState := SOFF;
     if assigned(impl) then begin
        impl.connect;
        TPlugin(messageEngine).setInstanceDescr(impl.getNiceName);
     end;
  end else begin
     addToLog('Already connected');
  end;
End;

procedure TfrmMain.onDisconnect;
Begin
   desiredState := DISCONNECTED;
   reconnState := SOFF;   
   if (connState=CONNECTED) and assigned(impl) then impl.disconnect;
End;

procedure TfrmMain.signalConnected;
Begin
   connState := CONNECTED;
   TPlugin(messageEngine).signalConnected
End;

procedure TfrmMain.signalNotConnected;
Begin
   connState := DISCONNECTED;
   TPlugin(messageEngine).signalNotConnected
End;

//********************************************************************
// Processing functions;
//********************************************************************

procedure TfrmMain.addtolog(event:string);
Begin
    TPlugin(messageEngine).sendInfo('PluginEvent',event);
End;

procedure TfrmMain.addtodirectlog(msg : widestring);
var filename : string;
    f:textfile;
Begin
    try
    filename := exedir+'Logs\'+instancename+'.direct.txt';
    assignfile(f,filename);
    if (fileexists(filename)) then append(f) else rewrite(f);
    Writeln(F,nowstr+' '+msg);
    CloseFile(F);
    except
    end;

End;

function TfrmMain.sendInfo(action,param:string):string;
Begin
   Result := TPlugin(messageEngine).sendInfo(action,param);
End;

end.
